

# Define some colors for outputs:
red='\e[0;31m'
RED='\e[1;31m'
blue='\e[0;34m'
BLUE='\e[1;34m'
cyan='\e[0;36m'
CYAN='\e[1;36m'
GREEN='\e[1;32m'
yellow='\e[1;33m'
fuchsia='\e[1;35m'
gray='\e[1;30m'
NC='\e[0m'              # No Color

NOTIFY_COLOR=$cyan
WARNING_COLOR=$fuchsia
ERROR_COLOR=$RED
QUESTION_COLOR=$GREEN

necho(){
	echo -e $NOTIFY_COLOR $@ $NC
}

eecho(){
	echo -e $ERROR_COLOR $@ $NC
}
qecho(){
	echo -e $QUESTION_COLOR $@ $NC
}
wecho(){
	echo -e $WARNING_COLOR $@ $NC
}
techo(){
	echo -e $@
}
startgecho(){
echo -ne $gray
}
stopgecho(){
echo -ne $NC
}


launch_cmd(){
	echo $@
	startgecho
	$@
	echo -ne $NC
}

ask(){
	qecho $@
	read ANSWER
}

ynask(){
	while true; do
		ask $@ "[y/n]"
		case $ANSWER in
			Y|y|S|s|yes) 
				ANSWER=y
				return 0
				;;
			N|n|no)
				ANSWER=n
				return 1
				;;
			*)
				wecho "Answer y or n"
				;;
		esac
	done
}

necho "Init variables"
launch_cmd source ./init_variables_scu.sh

necho "Compile tools"
launch_cmd java net.tinyos.signet.SecureSynapseInterface -compile-tools

necho "Deploy nodes"
techo "Connect the nodes you want to deploy, including the base station. Security parameters used are:\n 80 bits for authentication, 30 times keys duration;\n 128 bits for encryption;\n 128 bits for DoS protection"
ask "Press [enter] when ready"

launch_cmd java net.tinyos.signet.SecureSynapseInterface -deploy 80 30 128 128 -use-authentication -use-encryption -use-dos-protection

necho "Start dissemination"
techo "Disconnect all nodes and keep connected only the one used as the base station.\nIf more nodes are connected, the node with minimum serial number will be used as base station.\nYou will be asked the path of an already built application (the path must contain build/telosb/main.exe file) and an hexadecimal ID of the application"
ask "Press [enter] when ready"

while true; do 
	ask "Type the directory of the built application (containing build directory)"

	if [ ! -d "$ANSWER" ];then
		wecho "Directory $ANSWER does not exists"
		continue
	fi
	if [ ! -d "$ANSWER/build" ];then
		wecho "Directory $ANSWER/build does not exists"
		continue
	fi
	if [ ! -d "$ANSWER/build/telosb" ];then
		wecho "Directory $ANSWER/build/telosb does not exists"
		continue
	fi
	if [ ! -f "$ANSWER/build/telosb/main.ihex" ];then
		wecho "File $ANSWER/build/telosb/main.ihex does not exists"
		continue
	fi

	APP_DIR=$ANSWER
	break;
done

while true; do 
	ask "Type the desired application id (Hexadecimal, upper case characters)"

	DEC=$(echo "ibase=16;obase=A; $ANSWER" | bc 2>&1)
	echo "$DEC" | grep "standard_in"
	if [ $? -eq 0 ];then
		#error
		wecho "Invalid ID"
		continue
	fi

	HEX=$(echo "ibase=10;obase=16; $DEC" | bc 2>&1)
	if [ "$HEX" == "$ANSWER" ]; then
		echo -n ""
	else
		wecho "Invalid ID"
		continue
	fi
	ID=$ANSWER
	break
done

launch_cmd java net.tinyos.signet.SecureSynapseInterface -dissem $APP_DIR 800 $ID -format-bs -format-nodes -use-authentication -use-encryption -use-dos-protection









