/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef SYNAPSE__EVENTS_H
#define SYNAPSE__EVENTS_H

#include "Synapse_msgs.h"

enum{
    SYNEV_REQ_BLOCK=1,
    SYNEV_SENDING_BLOCK=2,
    SYNEV_RECEIVING_BLOCK=3,
    SYNEV_STOP_SENDING=4,
    SYNEV_STOP_RECEIVING=5,
    SYNEV_END_CYCLE=6,
    SYNEV_EXCH_ADV=7,
    SYNEV_EXCH_REQ=8,
    SYNEV_EXCH_STARTRX=9,
    SYNEV_EXCH_STOPRX=10,
    SYNEV_EXCH_STOPTX=11,
};

    Synapse_Event* getSynapse_EventMessage();
    void sendEvent(uint8_t type);

    void sendEventVoid(uint8_t type){
#ifdef SYNAPSE_EVENTS        
        sendEvent(type);
#endif      
    }
    
    void sendEventBlock(uint8_t type, uint8_t blockId) {
#ifdef SYNAPSE_EVENTS                       
        getSynapse_EventMessage()->params.blockParams.blockId=blockId;
        sendEvent(type);
#endif              
    }
    
    void sendEventEXCH(uint8_t type, uint16_t node_id, uint16_t cycle){
#ifdef SYNAPSE_EVENTS                       
        getSynapse_EventMessage()->params.exchParams.common.node_id=node_id;
        getSynapse_EventMessage()->params.exchParams.common.cycle=cycle;          
        sendEvent(type);
#endif              
    }    

void sendEventEXCH_ADV    (uint16_t node_id, uint16_t cycle, uint8_t hopCount, uint8_t slot){   
#ifdef SYNAPSE_EVENTS      
        getSynapse_EventMessage()->params.exchParams.params.exchADV.hopCount=hopCount;
        getSynapse_EventMessage()->params.exchParams.params.exchADV.slot=slot;
        sendEventEXCH(SYNEV_EXCH_ADV,node_id,cycle);
#endif      
}
void sendEventEXCH_REQ    (uint16_t node_id, uint16_t cycle, uint8_t block, uint8_t source, uint8_t slot){
#ifdef SYNAPSE_EVENTS     
        getSynapse_EventMessage()->params.exchParams.params.exchREQ.block=block;
        getSynapse_EventMessage()->params.exchParams.params.exchREQ.source=source;
        getSynapse_EventMessage()->params.exchParams.params.exchREQ.slot=slot;        
        sendEventEXCH(SYNEV_EXCH_REQ,node_id,cycle);
#endif      
}
void sendEventEXCH_STARTRX(uint16_t node_id, uint16_t cycle, uint8_t block, uint8_t source){
#ifdef SYNAPSE_EVENTS        
        getSynapse_EventMessage()->params.exchParams.params.exchSTARTRX.block=block;
        getSynapse_EventMessage()->params.exchParams.params.exchSTARTRX.source=source;
        sendEventEXCH(SYNEV_EXCH_STARTRX,node_id,cycle);
#endif      
}
void sendEventEXCH_STOPRX (uint16_t node_id, uint16_t cycle, uint8_t hopCount, uint8_t block, uint8_t pktrx, uint8_t remRank, int8_t rssi){
#ifdef SYNAPSE_EVENTS        
        getSynapse_EventMessage()->params.exchParams.params.exchSTOPRX.hopCount=hopCount;
        getSynapse_EventMessage()->params.exchParams.params.exchSTOPRX.block=block;
        getSynapse_EventMessage()->params.exchParams.params.exchSTOPRX.packets=pktrx;
        getSynapse_EventMessage()->params.exchParams.params.exchSTOPRX.remRank=remRank;
        getSynapse_EventMessage()->params.exchParams.params.exchSTOPRX.rssi=rssi;
        sendEventEXCH(SYNEV_EXCH_STOPRX,node_id,cycle);
#endif      
}   
void sendEventEXCH_STOPTX (uint16_t node_id, uint16_t cycle, uint8_t hopCount, uint8_t block){
#ifdef SYNAPSE_EVENTS     
        getSynapse_EventMessage()->params.exchParams.params.exchSTOPTX.block=block;
        getSynapse_EventMessage()->params.exchParams.params.exchSTOPTX.hopCount=hopCount;   
        sendEventEXCH(SYNEV_EXCH_STOPTX,node_id,cycle);
#endif      
}    

#endif /* SYNAPSE__EVENTS_H */
