/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef CC2420_SECURITY_H
#define CC2420_SECURITY_H

#include "CC2420.h"

#ifdef CC2420_HW_SECURITY

enum{
	CC2420_SEC_MODES=CCM_16+1,
};

/**
 * Translates secLevel in mode and micLength.
 * @returns TRUE if secLevel is valid, FALSE otherwise.
 */
inline bool getSecurityParams(uint8_t secLevel, uint8_t *mode, uint8_t *micLength){
    static const uint8_t modes  [CC2420_SEC_MODES]={CC2420_NO_SEC,CC2420_CBC_MAC,CC2420_CBC_MAC,CC2420_CBC_MAC,CC2420_CTR,CC2420_CCM,CC2420_CCM,CC2420_CCM};
    static const uint8_t lengths[CC2420_SEC_MODES]={0,4,8,16,0,4,8,16};
/*       	
    if       (secLevel >= CC2420_SEC_MODES){
      *mode = CC2420_NO_SEC;
      *micLength = 0;       
      return FALSE;
    }else{*/
    
  // secLevel should always be correct, if eliminated to save space  
      *mode      = modes  [secLevel];
      *micLength = lengths[secLevel];
      return TRUE;      
//    }    
}

void turnOnSecurity(message_t *msg, uint8_t secLevel, uint32_t nonce, uint8_t keyIndex){        
    cc2420_header_t* hdr = (cc2420_header_t*)msg->header;
    security_header_t* secHdr = (security_header_t*)&hdr->secHdr;
    secHdr->secLevel = secLevel;
    secHdr->keyMode = 1; // Fixed to 1 for now
    secHdr->reserved = 0; //skip in cc2420        
    secHdr->frameCounter = nonce;
    secHdr->keyID[0] = keyIndex; // Always first position for now due to fixed keyMode
    hdr->fcf |= 1 << IEEE154_FCF_SECURITY_ENABLED;      
}

bool packetIsAuthentic(message_t *msg){
	uint8_t mode;
	uint8_t micLength;
	cc2420_header_t* hdr = (cc2420_header_t*)msg->header;
    security_header_t* secHdr = (security_header_t*)&hdr->secHdr;
    
	if((hdr->fcf & (1 << IEEE154_FCF_SECURITY_ENABLED))==0)
	   return FALSE; // Security not enabled
	getSecurityParams(secHdr->secLevel,&mode,&micLength);
	if((mode==CC2420_CTR)||(mode==CC2420_NO_SEC))
	   return FALSE; // Authentication not enabled
	return (((uint8_t*)msg)[msg->header[0]-2]==0);
}

#endif

#endif /* CC2420_SECURITY_H */
