/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

package net.tinyos.signet;

import java.util.Arrays;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.engines.AESEngine;
import org.bouncycastle.crypto.modes.OFBBlockCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.params.ParametersWithIV;
import org.bouncycastle.util.encoders.Hex;

public class DaviesMeyerHashConstruction implements HashFunction{

	BlockCipher bc;
	
	public DaviesMeyerHashConstruction(BlockCipher aBlockCypher){
		this.bc=aBlockCypher;		
	}
	
	@Override
	public byte[] calculateHash(byte[] message, int length) {
		int blockSize=bc.getBlockSize();
		int remaining=message.length;
		int srcPos=0;
		
		byte[] result=new byte[blockSize];
		byte[] key   =new byte[blockSize];
		byte[] iv    =new byte[blockSize];

		Arrays.fill(iv, (byte)0);

		while(remaining>0){
			int len=Math.min(blockSize, remaining);
			Arrays.fill(key, (byte)0);
			System.arraycopy(message, srcPos, key, 0, len);
			srcPos+=len;
			
			bc.init(true, new KeyParameter(key));		
							
			bc.processBlock(iv, 0, result, 0);
			
			for(int i=0;i<blockSize;i++)
				iv[i]^=result[i];
			
			remaining-=len;
		}		
		return Arrays.copyOf(iv,length);
	}

	@Override
	public byte[] calculateHash(byte[] message, int length, int hashIterations) {
		byte[] result = calculateHash(message, length);

		for(int i=0;i<hashIterations;i++){
			result = calculateHash(result, length);
		}
		
		return result;		
	}

	public static void main(String[] args) {
		HashFunction hf=new DaviesMeyerHashConstruction(new AESEngine());
		//byte[] message=Hex.decode("24CAB29B81BF5AB8A5E0");
		byte[] message=new byte[800];
		//Arrays.fill(message, (byte)0);
		System.out.println(Arrays.toString(message));
		System.out.println(Arrays.toString(hf.calculateHash(message, 10)));
	}
	
}
