/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

package net.tinyos.signet;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

public abstract class KeyRing<keyClass extends Key> {
	protected int keyNum;
	protected ArrayList<keyClass> keyList;
	
	public KeyRing() {
		this.keyNum=0;
		keyList=new ArrayList<keyClass>();
	}
	
	public KeyRing(int count) {
		this.keyNum=count;
		keyList=new ArrayList<keyClass>(keyNum);
	}
	
	protected keyClass getKey(int index){
		return keyList.get(index);
	}		
	
	protected abstract keyClass createKey(int index);
	
	public void saveToXML(Element container) throws ParserConfigurationException{
		Document doc=container.getOwnerDocument();
		Element ecount =doc.createElement("keyCount");
		container.appendChild(ecount);
		ecount .appendChild(doc.createTextNode(Integer.toString(keyNum)));
		
		for(int i=0;i<keyNum;i++){
			Element ekey=doc.createElement("key");
			container.appendChild(ekey);
			keyList.get(i).saveToXML(ekey);
		}
	}
	public void loadFromXML(Element container) throws ParserConfigurationException{
		Document doc=container.getOwnerDocument();
		keyNum=Integer.parseInt(
				container.getElementsByTagName("keyCount").item(0).getTextContent()
				);
		keyList.clear();
		NodeList ekeys=container.getElementsByTagName("key");
		for(int i=0;i<ekeys.getLength();i++){
			Node ekey =ekeys.item(i);
			keyList.add(createKey(i));
			keyList.get(i).loadFromXML((Element)ekey);
		}							
	}
		
	private static class TestKeyRing extends KeyRing<Key>{
		@Override
		protected Key createKey(int index) {
			return new Key(index);
		}		
	}
	
	public static void main(String[] args) throws ParserConfigurationException, IOException {
		KeyRing<Key> kr=new TestKeyRing();
		for(int i=0;i<10;i++){
			kr.keyList.add(new Key(i,10));
			kr.keyList.get(i).value[0]=(byte)i;
		}
		kr.keyNum=10;
		
		Document doc =DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		Element root=doc.createElement("Main");
		doc.appendChild(root);
				
		kr.saveToXML(root);
		
		XMLSerializer xs=new XMLSerializer();
		xs.setOutputCharStream(new OutputStreamWriter(System.out));
		xs.serialize(doc);	
		
		kr=null;
		kr=new TestKeyRing();
		kr.loadFromXML(root);
		
		doc =DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		root=doc.createElement("Main");
		doc.appendChild(root);
				
		kr.saveToXML(root);
		
		xs=new XMLSerializer();
		xs.setOutputCharStream(new OutputStreamWriter(System.out));
		xs.serialize(doc);			
	}
}
