/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

package net.tinyos.signet;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

public class KeyStorage {
	private String filename;
	private Document doc;
	private ArrayList<String> ids=new ArrayList<String>();
	private Hashtable<String, KeyRing> ht=new Hashtable<String, KeyRing>();
	
	public KeyStorage(String filename) throws SAXException, IOException, ParserConfigurationException{
		this.filename=filename;
		File f=new File(filename);
		if(f.exists()){
			doc=DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(filename);
		}else{
			doc =DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
			Element root=doc.createElement("net.tinyos.signet.KeyStorage");
			doc.appendChild(root);
			writeDocument();
		}
	}
	
	private void writeDocument() throws IOException{
		XMLSerializer xs=new XMLSerializer();
		xs.setOutputCharStream(new FileWriter(filename));
		xs.serialize(doc);		
	}
	
	public void addKeyRing(String id,KeyRing kr){
		ids.add(id);
		ht.put(id, kr);		
	}
	
	public KeyRing getKeyRing(String id){		
		return ht.get(id);		
	}
	
	public Element getKeyRingContainer(String id){		
		return (Element)doc.getElementsByTagName(id).item(0);
	}
	
	public void saveToXML(Document doc, Element container) throws ParserConfigurationException{		
		for(int i=0;i<ids.size();i++){
			NodeList nl=doc.getElementsByTagName(ids.get(i));
			if(nl.getLength()>0){
				Node n=nl.item(0);
				doc.getDocumentElement().removeChild(n);
			}
			Element ekey=doc.createElement(ids.get(i));
			container.appendChild(ekey);
			ht.get(ids.get(i)).saveToXML(ekey);
		}
	}	
	
	public void save() throws ParserConfigurationException, IOException{
		saveToXML(doc, doc.getDocumentElement());
		writeDocument();
	}
	
	private static class TestKeyRing extends KeyRing<Key>{
		@Override
		protected Key createKey(int index) {
			return new Key(index);
		}		
	}	
	
	public static void main(String[] args) throws ParserConfigurationException, IOException, SAXException {
		KeyStorage ks=new KeyStorage("/home/dissegn1/synapse-keystorage.xml");
		
		KeyRing<Key> kr=new TestKeyRing();
		kr.keyNum=7;
		for(int i=0;i<kr.keyNum;i++){
			kr.keyList.add(new Key(i,10));
		}		
		ks.addKeyRing("prova", kr);
		
		kr=new TestKeyRing();
		kr.keyNum=7;
		for(int i=0;i<kr.keyNum;i++){
			kr.keyList.add(new Key(i,10));
		}				
		ks.addKeyRing("prova1", kr);		
		
		ks.save();
	}
}
