/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

package net.tinyos.signet;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.BitSet;


public class SHA1HashFunction implements HashFunction{
	@Override
	public byte[] calculateHash(byte[] message,int length){
	    byte[] sha1hash = new byte[40];
		MessageDigest md;
		try {
			md = MessageDigest.getInstance("SHA-1");
			md.update(message, 0, message.length);
			sha1hash = md.digest();
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		 
		sha1hash=Arrays.copyOfRange(sha1hash, sha1hash.length-length, sha1hash.length);
		
		return sha1hash;
	}
	@Override
	public byte[] calculateHash(byte[] message,int length,int hashIterations){
		byte[] result = calculateHash(message, length);

		for(int i=0;i<hashIterations;i++){
			result = calculateHash(result, length);
		}
		
		return result;		
	}
	
	
		
	public static void main(String[] args) {
		

// Test per vedere il numero minimo di uni che si riesce ad ottenere nell'hash
   
		SHA1HashFunction sha1=new SHA1HashFunction();
		
		byte msg[]=new byte[100000],hash[];
		
		for(int i=0;i<msg.length;i++)
			//msg[i]=(byte)i;
			msg[i]=(byte)(Math.random()*255);		
		
		
		int oneCount,minj=0,minOnes=80;
		for(int j=0;j<255;j++){
			msg[0]=(byte)(j);
			
			hash=sha1.calculateHash(msg,10);
				
			BitSet bs=BitSetUtils.convertToBitSet(hash);
			
			oneCount=0;			
			for(int i=0;i<bs.cardinality();i++)
				oneCount++;
			
			if(oneCount<minOnes){
				minj=j;
				minOnes=oneCount;
			}
		}
		
		msg[0]=(byte)(minj);		
		hash=sha1.calculateHash(msg,10);			
		BitSet bs=BitSetUtils.convertToBitSet(hash);
		
		oneCount=0;			
		for(int i=0;i<bs.cardinality();i++)
			oneCount++;		
		
		for(int i=0;i<hash.length;i++)
			System.out.print(""+((short)hash[i])+",");		
		System.out.println();		
		System.out.println(bs);
		System.out.print(""+oneCount+" ones");
/**/		
		
/*		
// 	Test per verificare la coerenza tra le implementazioni java e c	
	
		SHA1HashFunction sha1=new SHA1HashFunction();
		
		byte msg[]=new byte[100],hash[];
		
		for(int i=0;i<msg.length;i++)
			msg[i]=(byte)i;	
		
		
		
		hash=sha1.calculateHash(msg,10);					
		System.out.println(Arrays.toString(hash));
		hash=sha1.calculateHash(msg,10,2);
		System.out.println(Arrays.toString(hash));
		
/**/			
	}


	 
}
