/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

package net.tinyos.signet;

import java.io.IOException;
import java.util.concurrent.Semaphore;


import net.tinyos.message.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;

public class Suino implements MessageListener{
	
	private static final int AM_SIGNETNP_SERIALCOMMAND 	= 127;

	
	private static final short transferCmd	= 1;
	private static final short netFormatCmd	= 2;
	private static final short netResetCmd	= 3;
	private static final short netLoadCmd		= 4;
	private static final short tagCmd			= 9;
	private static final short advProcedure	= 5;
	private static final short prepareCmd		= 6;
	private static final short pingReqMsg		= 7;
	private static final short pingRepMsg		= 8;
	private static final short aliveMsg		= 10;	
	
	private static final short fbDisseminationComplete=1;
	private static final short fbAlive=2;
	
	private Object semDisseminationComplete=new Object();
	private Object semAliveRx=new Object();
	
	  private MoteIF moteIF;
	  
	  public Suino(MoteIF moteIF) {
	    this.moteIF = moteIF;
	    this.moteIF.registerListener(new SynapseSerialCmdMessage(), this);
	    this.moteIF.registerListener(new SynapseSerialFeedbackMessage(), this);
	  }
	  
	  public void sendCommand(short cmd,int arg) {
        SynapseSerialCmdMessage payload = new SynapseSerialCmdMessage();
	    
	    try {      
		
		payload.set_cmd(cmd);
		payload.set_arg(arg);
		moteIF.send(0, payload);
		
	    }
	    catch (IOException exception) {
	      System.err.println("Exception thrown when sending packets. Exiting.");
	      System.err.println(exception);
	    }
	  }

	  public void messageReceived(int to, Message message) {
		  if (message instanceof SynapseSerialFeedbackMessage){
			  SynapseSerialFeedbackMessage fb=(SynapseSerialFeedbackMessage)message;
			  if(fb.get_type()==fbDisseminationComplete){
				  synchronized(semDisseminationComplete){
					  semDisseminationComplete.notify();
				  }
			  }else if (fb.get_type()==fbAlive){
				  synchronized(semAliveRx){
					  semAliveRx.notify();
				  }				  
			  }
		  }
	  }
	  
	  public void waitDissemination() throws InterruptedException{
		  synchronized(semDisseminationComplete){
			  semDisseminationComplete.wait();
		  }		  
	  }
	  public void waitAliveRx() throws InterruptedException{
		  synchronized(semAliveRx){
			  semAliveRx.wait();
		  }		  
	  }	  
	  
	  private static void usage() {
	    System.err.println("usage: Suino [-comm <source>]");
	  }
	  
	  public static void main(String[] args) throws Exception {
		  
		  String source=null;
		  
		  
			String arg;
			int argi=0;
			while(argi<args.length){
				arg=args[argi];
				if      (arg.equals("-comm")){
					argi++;
					source=args[argi];
				}
				argi++;
			}			  
		  
	    
		    PhoenixSource phoenix;
		    
		    if (source == null) {
		      phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);
		    }
		    else {
		      phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
		    }

		    MoteIF mif = new MoteIF(phoenix);
		    
		    Suino serial = new Suino(mif);
	    
			argi=0;
			while(argi<args.length){
				arg=args[argi];
				if      (arg.equals("-comm")){
					argi++;
				}else if(arg.equals("--prepare")){
					serial.sendCommand(prepareCmd,0);
					System.exit(0);
				}else if (arg.equals("--format")){
					serial.sendCommand(netFormatCmd,0);
					System.exit(0);
				}else if (arg.equals("--reset")){
					serial.sendCommand(netResetCmd,0);
					System.exit(0);
				}else if (arg.equals("--load")){
					serial.sendCommand(netLoadCmd,Integer.parseInt(args[++argi], 16));
					System.exit(0);
				}else if (arg.equals("--transfer")){
					serial.sendCommand(transferCmd,Integer.parseInt(args[++argi], 16));
					serial.waitDissemination();
					System.exit(0);
				}else if (arg.equals("--alive")){
					serial.sendCommand(aliveMsg,0);
					serial.waitAliveRx();
					System.exit(0);
				}else{
					System.err.println("Invalid argument: "+arg+"\n");
					usage();
					System.exit(1);				
				}
				argi++;
			}	    
	    
	    
	    
	  }


	
	




	
	
/*

#COMMANDS


class SerialPingPacket(tinyos.GenericPacket):
	def __init__(self, packet = None):
		tinyos.GenericPacket.__init__(self,
		[('dest','int',  2),
		('partNum', 'int',  1),
		('pingFlags',   'int',  1),
		('status',   'int',  1),
		('imgSize',   'int',  4),
		('imgOffset',   'int',  4),
		('imgId',   'int',  2),
		('imgFlags',   'int',  2)],
		packet)

class ReplyPacket(tinyos.GenericPacket):
	def __init__(self, packet = None):
		tinyos.GenericPacket.__init__(self,
		[('type', 'int',  1),
		('size',  'int',  4),
		('p32_1',    'int',  4),
		('appID', 'int',  2),
		('flags', 'int',  2),
		('pingFlags',  'int',  1),
		('status','int',  1),],
		packet)
		
class CommandPacket(tinyos.GenericPacket):
	def __init__(self, packet = None):
		tinyos.GenericPacket.__init__(self,
		[('cmd', 'int',  1),
		('arg',  'int',  2),],
		packet)

def parseStatus(s):
	if (s==0):
		return "Initializing"
	if (s==1):
		return "Idle"
	if (s==2):
		return "Preparing to execute a command"
	if (s==3):
		return "Adv Phase"
	if (s==4):
		return "Req Phase"
	if (s==5):
		return "Data Phase"


def int16ToStr(n):
	return chr(int(math.floor(n/256)))+chr(int(n-(math.floor(n/256))*256))



def ping(nodeID,appNum,port=9000):
	if (port!=0):
		comm=tinyosSF.Forwarder(port)
	else:
		comm=tinyos.Serial(serialPort, platform)
	pe="";
	spingpkt = SerialPingPacket((nodeID, appNum, 0,0,0,0,0,0))
	comm.write_packet(0, AM_SIGNETNP_SERIAL_PING, spingpkt.payload())
	print "Ping req sent to node %d" %nodeID  + " for partition #%d" %appNum
	while True:
		packet=comm.read_packet(0,AM_SIGNETNP_REPLY)
		replyPkt=ReplyPacket(packet[1]);
		if(replyPkt['type']==8):
			status = parseStatus(replyPkt['status'])
			print "Node %d is alive" %nodeID + " and " + status
			if((replyPkt['pingFlags'])&PING_PREPARED_FLAG):
				prepared="set"
			else:
				prepared="not set"
			print "Prepared flag is " + prepared
			if((replyPkt['pingFlags'])&PING_OVERFLOW_FLAG):
				print "Slot %d does not exist " %appNum
				print "%d slots are available" %replyPkt['appID']
				print "%d slots are used" %replyPkt['size']
				break
			else:
				if(replyPkt['appID']!=0xFFFF):
					if ((replyPkt['flags']^0xFFFF)&SPE_WORKING):
						print "Application in slot %d has ID " %appNum + " %X" %replyPkt['appID']
					else:
						print "Slot %d occupied by NON WORKING Application" %appNum
						print "Application stored is %X" %replyPkt['appID'] + " (Non Working)"
					print "Application size is %d bytes" %replyPkt['size']
				else:
					print "Slot %d is free" %appNum
				if ((replyPkt['flags']^0xFFFF)&SPE_USED):
					print "Slot %d is marked USED" %appNum
				else:
					print "Slot %d is marked UNUSED" %appNum
				break

def printUsage():
	print "SUINO (Synapse User Interface Network Operation) "
	print "usage: suino [options] --ping nodeAddr PartNum"
	print "or suino --transfer id"
	print "or suino --load id"
	print "or suino --prepare "
	print "or suino --format "
	print "or suino --reset "
	print "or suino --tag tag"
	print "options are: -c serial_Port , --port TCP_port, --platform baud_rate"

if (len(sys.argv)<2):
	printUsage()
for j in range(len(sys.argv)):
	if (sys.argv[j]=='-c'):
		serialPort=sys.argv[j+1]
	if (sys.argv[j]=='-p'):
		platform=sys.argv[j+1]
	if (sys.argv[j]=='--port'):
		port=int(sys.argv[j+1])
	if (sys.argv[j]=='--help'):
		printUsage()
for j in range(len(sys.argv)):
	if (sys.argv[j]=='--prepare'):
		sendCmd(prepareCmd,0,port)
	if (sys.argv[j]=='--format'):
		sendCmd(netFormatCmd,0,port)
	if (sys.argv[j]=='--reset'):
		sendCmd(netResetCmd,0,port)
	if (sys.argv[j]=='--load'):
		sendCmd(netLoadCmd,string.atoi(sys.argv[j+1],16),port)
	if (sys.argv[j]=='--tag'):
		sendCmd(tagCmd,string.atoi(sys.argv[j+1],16),port)
	if (sys.argv[j]=='--transfer'):
		sendCmd(transferCmd,string.atoi(sys.argv[j+1],16),port)
	if (sys.argv[j]=='--ping'):
		ping(int(sys.argv[j+1]),int(sys.argv[j+2]),port)



 */

}
